//
//  fairness_utilities.c
//  
//
//  Created by Ivaylo Dobrikov on 21/02/14.
//
//

#include <stdio.h>

#include "fairness_utilities.h"

///*
// * The returned list term has the form:
// *  "[anode(AtomId,Nodeid,[trans(NextAtomId,NextNodeId,[TransitionId])])]"
// */
//
//SP_term_ref get_scc_as_list(tarjandata *t, int scc_size) {
//    ltl_formula *f = t->f;
//    SP_atom tfunctor = SP_atom_from_string("trans");
//    SP_atom afunctor = SP_atom_from_string("anode");
//    SP_term_ref list = SP_new_term_ref();
//    SP_term_ref tail,head,nodeid,transitions,atomid_term;
//    
//    // save the found SCC in prolog list form (result will be saved in the variable "list")
//    int i,atomid;
//    for (i=scc_size; i>0; i--) {
//        ltlatom *atom = peek_stack(t,i-1);
//        atomid = find_atom_id(f,atom);
//        nodeid = get_node_id(atom->node);
//        transitions = get_outgoing_edges(f,tfunctor,atom,atomid);
//        head = SP_new_term_ref();
//        atomid_term = SP_new_term_ref();
//        SP_put_integer(atomid_term, atomid);
//        SP_cons_functor(head, afunctor, 3, atomid_term, nodeid, transitions);
//        tail = list;
//        list = SP_new_term_ref();
//        SP_cons_list(list, head, tail);
//    }
//    return list;
//}
//
///*
// * returns all outgoing transitions from an atom as a Prolog list.
// * the list has the form "functor(NextAtomId,NextNodeId,[TransitionId])"
// */
//SP_term_ref get_outgoing_edges(ltl_formula *f, SP_atom functor, ltlatom *atom, unsigned int atomid) {
//    ltlatom *succ;
//    SP_term_ref list = SP_new_term_ref();
//    SP_term_ref tail, transitions, head, succid_term;
//    int succid,nodeid;
//    int i;
//    unsigned int first_tid = atom->first_transition_id;
//    for (i=0; i<atom->number_of_transitions; i++) {
//        
//        succ = get_atom_trans(f,first_tid+i);
//        succid = find_atom_id(f,succ);
//        nodeid = get_node_id(succ->node);
//        transitions = find_transition_operations(f,atom,succ);
//        
//        succid_term = SP_new_term_ref();
//        SP_put_integer(succid_term,succid);
//        
//        head = SP_new_term_ref();
//        SP_cons_functor(head, functor, 3, succid_term, nodeid, transitions);
//        tail = list;
//        list = SP_new_term_ref();
//        SP_cons_list(list, head, tail);
//    }
//    return list;
//}

int must_check_fairness(tarjandata *t) {
    return t->f->check_found_scc_for_fairness_activated;
}

/*
 * Create a Prolog list that contains all IDs of the nodes contained in the SCC.
 * (Because there can be multiple atoms in the SCC that correspond to a single node,
 * a node can be contained in the list several times.)
 */
SP_term_ref create_scc_nodes_list(tarjandata *t, int scc_size) {
    ltlatom *atom;
    ltlnode *node;
    SP_term_ref ref,list,tail;
    unsigned int i;
    tail = SP_new_term_ref();
    list = tail;
    if (t->scc_stacksize >= scc_size) {
        for (i=0; i<scc_size; i++) {
            atom = peek_stack(t, i);
            node = atom->node;
            if (node->type == int_node) {
                ref = SP_new_term_ref();
                SP_put_integer(ref, node->id.id);
            } else {
                ref = node->id.ref;
            }
            list = SP_new_term_ref();
            SP_cons_list(list,ref,tail);
            tail = list;
        }
    } else {
        fprintf(stderr, "save_stack(): scc stack underflow!!");
        exit(1);
    }
    return list;
}

void print_scc_state_ids(SP_term_ref scc_state_ids) {
    SP_term_ref
	tail = SP_new_term_ref(),
	head = SP_new_term_ref();
   SP_put_term(tail,scc_state_ids);
   SP_integer i;
   printf("[");
   while (SP_get_list(tail,head,tail)) {
      SP_get_integer(head,&i);
      printf(" %d ", (int) i);
   }
   printf("]\n");
}

/*********** Functions used for determinining the fairness of loops ***********/

int check_if_loop_contains_transitions(ltl_formula *f, ltlatom **loop_atoms, int loop_size, SP_integer *matching_transitions, int msize) {
    
    if (loop_size>1) {
        ltlatom *src, *dst;
        int i=0;
        while (i+1 < loop_size) {
            src = loop_atoms[i];
            dst = loop_atoms[i+1];
            if (find_transition_operation_that_matches(f,src,dst,matching_transitions,msize,NULL)){
                return 1;
            }
            i++;
        }
        return 0;
    } else {
        fprintf(stderr, "Checking fairness of loop. Loop array must have minimum two elements.");
        exit(1);
    }
}

/*********** Functions used for determinining the fairness of SCCs ***********/
/*
 * Check if there is an internal transition in the SCC that matches the fairness criteria.
 * The list of matching transitions is given by an array of transition ids (msize is the
 * array's size).
 */
int check_if_scc_contains_transitions1(ltl_formula *f, ltlatom **scc, int scc_size, SP_integer *matching_transitions, int msize) {
    /* TODO: Maybe the sorted version of the SCC can be generated in a more central
     position. */
    ltlatom *src,*dst;
    unsigned int i,j,first;
    /* Iterate through the SCC */
    if (msize > 0) {
        for (i=0; i<scc_size; i++) {
            src = scc[i];
            first = src->first_transition_id;
            /* Iterate through the atom's outgoing transitions */
            for (j=0; j<src->number_of_transitions; j++) {
                dst = get_atom_trans(f,first+j);
                /* Check if the destination atom is also in the SCC */
                if (is_in_scc(dst,scc_size,scc) &&
                    /* and one of the transitions between the atoms is matching the fairness criteria */
                    find_transition_operation_that_matches(f,src,dst,matching_transitions,msize,NULL)) {
                    return 1;
                }
            }
        }
    }
    /* No transition found. */
    return 0;
}

int check_if_scc_contains_transitions(tarjandata *t, int scc_size, SP_integer *matching_transitions, int msize) {
    
    ltlatom *entry;
    ltlatom **scc = save_stack(t,scc_size, &entry,1,0);
    ltl_formula *f = t->f;

    int result = check_if_scc_contains_transitions1(f,scc,scc_size,matching_transitions,msize);
    ltl_free(scc);
    return result;
}

static int transition_operation_enabled_in_node(ltl_formula *f,ltlatom *src,SP_integer *matching_transitions, int msize) {
    
    unsigned int j,first,nr;
    SP_term_ref op;
    ltlnode *node;
    
    node = src->node;
    nr = node->number_of_transitions;
    first = node->first_transid;
    /* Iterate through the node's outgoing transitions */
    if (msize >0) {
        for (j=0; j<nr; j++) {
            get_nodetransop(f,first+j,NULL,NULL,&op);
            if (transition_in_array(op,matching_transitions,msize)) {
                return 1;
            }
        }
    }
    
    return 0;
}

// testing whether the operation is enabled in each node of the SCC
int enabled_in_each_node1(ltl_formula *f, ltlatom **scc, int scc_size, SP_integer *matching_transitions, int msize) {
    
    ltlatom *src;
    unsigned int i;
    
    /* Iterate through the SCC */
    if (msize > 0) {
        for (i=0; i<scc_size; i++) {
            src = scc[i];
            /* Iterate through the atom's outgoing transitions */
            if (!transition_operation_enabled_in_node(f,src,matching_transitions,msize)) {
                IFFAIRNESSDEBUG(ltl_printnode_with_info("src (enabled_in_each_atom): ",src->node););
                return 0;
            }
        }
    }
    /* enabled everywhere. */
    return 1;
}

int enabled_in_each_node(tarjandata *t, int scc_size, SP_integer *matching_transitions, int msize) {

    ltlatom *entry;
    ltlatom **scc = save_stack(t,scc_size, &entry,1,0);
    ltl_formula *f = t->f;

    int result = enabled_in_each_node1(f,scc,scc_size,matching_transitions,msize);
    ltl_free(scc);
    return result;
}

// testing whether the operation is enabled in some node of the SCC
int enabled_in_some_node1(ltl_formula *f, ltlatom **scc, int scc_size, SP_integer *matching_transitions, int msize) {

    ltlatom *src;
    unsigned int i;

    /* Iterate through the SCC */
    if (msize > 0) {
        for (i=0; i<scc_size; i++) {
            src = scc[i];
            IFFAIRNESSDEBUG(ltl_printnode_with_info("src (enabled_in_some_atom): ",src->node););
            /* Iterate through the atom's outgoing transitions */
            if (transition_operation_enabled_in_node(f,src,matching_transitions,msize)) {
                return 1;
            }
        }
    }
    /* enabled everywhere. */
    return 0;
}

int enabled_in_some_node(tarjandata *t, int scc_size, SP_integer *matching_transitions, int msize) {

    ltlatom *entry;
    ltlatom **scc = save_stack(t,scc_size, &entry,1,0);
    ltl_formula *f = t->f;

    int result = enabled_in_some_node1(f,scc,scc_size,matching_transitions,msize);
    ltl_free(scc);
    return result;
}

/*
 * Computes all atoms in an SCC where one of the transitions in "matching_transitions"
 * is enabled.
 * The result is stored in runfair as a newly allocated array with rsize as it's size.
 */
void extract_unfair_atoms(tarjandata *t, int scc_size, SP_integer *matching_transitions, int msize, ltlatom ***runfair, int *rsize) {
    /* TODO: Maybe the sorted version of the SCC can be generated in a more central
     position. */
    ltlatom* entry;
    ltlatom **scc = save_stack(t,scc_size,&entry,1,0);
    ltl_formula *f = t->f;
    ltlnode *node;
    unsigned int i,j,first,nr,size;
    SP_term_ref op;
    size = 0;
    ltlatom **unfair = 0;
    /* Iterate through the SCC */
    if (msize > 0) {
        /* TODO: High memory usage by just allocating the maximum, maybe it is more effective
         * to determine the size of the array first or use another kind of data structure */
        unfair = (ltlatom **) ltl_malloc(sizeof(ltlatom*) * scc_size);
        for (i=0; i<scc_size; i++) {
            node = scc[i]->node;
            nr = node->number_of_transitions;
            first = node->first_transid;
            /* Iterate through the node's outgoing transitions */
            for (j=0; j<nr; j++) {
                get_nodetransop(f,first+j,NULL,NULL,&op);
                if (transition_in_array(op,matching_transitions,msize)) {
                    unfair[size] = scc[i];
                    size += 1;
                }
            }
        }
    }
    ltl_free(scc);
    *runfair = unfair;
    *rsize = size;
}

// int operation_executed_in_scc(tarjandata *t, int scc_size, SP_term_ref match_op) {
//   ltlatom* entry;
//   ltlatom **scc = save_stack(t,scc_size,&entry,1,0);
//   ltl_formula *f = t->f;
//   ltlnode *node;
//   SP_term_ref op;
//   ltlatom* dst;
//   unsigned int i,j,nr,first;
//   int res = 0;
//   for(i=0; i<scc_size; i++) {
//       node = scc[i]->node;
//       nr = node->number_of_transitions;
//       first = node->first_transid;
//       for(j=0; j<nr; j++) {
//          get_nodetransop(f,first+j,NULL,NULL,&op);
//          dst = get_atom_trans(f,first+j);
//          /* Check if the destination atom is also in the SCC and
//             compare both operation terms op and match_op.
//             if op == match_op the SP_compare(op,match_op) returns 0 */
//          if(is_in_scc(dst,scc_size,scc) && SP_compare(match_op,op) == 0) 
//             res = 1;
//      }
//   }
//   ltl_free(scc);
//   return res;
// }

// SP_term_ref* always_enabled_operations(tarjandata *t, int scc_size, int *term_ref_size) {
//   ltlatom* entry;
//   ltlatom **scc = save_stack(t,scc_size,&entry,1,0);
//   ltl_formula *f = t->f;
//   ltlnode *node;
//   SP_term_ref* always_ops;
//   SP_term_ref op, op_entry;
//   unsigned int i,j,k,nr,first_entry,first,max_ops,always_size;
//   // we look for ops that are always enabled;
//   // the maximum is bound by the number of enabled ops in the atom with the least enabled ops
//   max_ops = (entry->node)->number_of_transitions;
//   first_entry = (entry->node)->first_transid;
//   always_size = 0;
//   always_ops = (SP_term_ref *) ltl_malloc(sizeof(SP_term_ref) * max_ops);
//   for(i=0; i<max_ops; i++) {
//      get_nodetransop(f,first_entry+i,NULL,NULL,&op_entry);
//      ltl_printterm_with_info("op_entry: ", op_entry);
//      // test if op is enabled in the rest of the atoms
//      int times_enabled=0;
//      for(j=0; j<scc_size; j++) {
//        node = scc[i]->node;
//        nr = node->number_of_transitions;
//        first = node->first_transid;
//        for(k=0; k<nr; k++) {
//          get_nodetransop(f,first+k,NULL,NULL,&op);
//          ltl_printterm_with_info("op: ", op);
//          // compare both operation terms op and op_entry.
//          // if op == op_entry the SP_compare(op,op_entry) returns 0 
//          if(SP_compare(op_entry,op) == 0) {
//             times_enabled++;
//             break;
//          }
//        }
//      }
//      // op_entry is enabled every atom of the SCC
//      if(times_enabled==scc_size) {
//        always_ops[always_size] = op_entry;
//        always_size++;
//      }  
//   }
//   *term_ref_size = always_size;
//   return always_ops;
// }

// int is_weak_fair_for_all_operations(tarjandata *t, int scc_size) {
//   int always_enabled_size;
//   SP_term_ref* always_enabled_ops = always_enabled_operations(t, scc_size, &always_enabled_size);
//   int i,res=1;
//    printf("always_enabled_size = %d\n", always_enabled_size);
//   for(i=0; i<always_enabled_size; i++) {
//      if (!operation_executed_in_scc(t, scc_size, always_enabled_ops[i])) {
//         res=0;
//         break;
//      }
//   }
//   ltl_free(always_enabled_ops);
//   return res;
// }


