/* 
* work with static memory allocation for the labels etc.
*/

//includes nauty.h 
#include "interface.h"

int main(int argc, char *argv[])
{

// example run
int exists = 0; 
prob_init(); 
prob_get_number_of_colours(230); 

prob_start_graph(2); 
prob_set_colour_of_node(0, 0); 
prob_set_colour_of_node(1, 17); 
prob_add_edge(1, 0); 
prob_exists_graph();

prob_start_graph(40); 
prob_set_colour_of_node(0, 5); 
prob_add_edge(0, 8); 
prob_set_colour_of_node(8, 6); 
prob_add_edge(28, 18); 
prob_add_edge(28, 38); 
prob_add_edge(28, 22); 
 
prob_start_graph(50); 
prob_set_colour_of_node(0, 0); 
prob_add_edge(0, 10); 
prob_set_colour_of_node(10, 1); 
prob_add_edge(10, 0); 
prob_add_edge(10, 20); 
prob_set_colour_of_node(20, 2); 
prob_add_edge(20, 10); 
prob_add_edge(20, 30); 
prob_set_colour_of_node(30, 3); 
prob_exists_graph();
 
prob_free_storage();

return 0;
}



/* functions for testing and debugging
*/

/* initialize char array with 0's
 * used as one row in adjacency matrix for small n (n<WORDSIZE)
 * matrix must store WORDSIZE elements plus one end sign
 */
void initArray(char matrix[]){

	int i;
	for(i=0; i<WORDSIZE; i++){  // initialise array with 0's 
		matrix[i] = '0' ;
	}
	matrix[i] = '\0' ; // mark end of row with string terminating character
}


/* print adjacency matrix of graph g with n nodes  */
void printGraph(graph *g, int n){
	
	char adjmat[WORDSIZE+1]; // end sign '\0'  store one set
	int k =0;
	int m = (n + WORDSIZE - 1) / WORDSIZE;

	printf("\n");

	while(k < n*m){ // for every node g has m sets that need to be considered as one row

		printf("%2d ", k/m); // print row number
		int l=0;
		for(l=0; l<m; l++){ //print m sets, which represent one row in adjacency matrix
			initArray(adjmat);
			intToBin(g[k+l], adjmat);

			int j = 0;
			
			while( adjmat[j] != '\0'){ //j<n &&
				printf(" %c", adjmat[j]) ;	//print each graphrow -> matrix
				j++;
			}
		}
		printf("\n");
		k = k+m;
	}
	printf("\n");
}

void print_canon_tree(struct Node *treeNode){
	//printf("print_canon_tree:\n");
	if(treeNode){	
		int n = treeNode->number_of_nodes;
		//printf("%u \n", *(treeNode->label)); //label is pointer
		printf("number_of_nodes: %d\n", n);
		printf("tree_node_number: %d\n", treeNode->tree_node_number);
		//array of cell sizes has global_number_of_colours  entries
		print_cell_sizes(treeNode->cell_sizes, global_number_of_colours);
		printGraph(treeNode->label, n);
		print_canon_tree(treeNode->left);
		print_canon_tree(treeNode->right);	
	}
}

void print_cell_list(int number_of_nodes, int number_of_colours, int *cell_list){

	int j;
	for(j=0; j<number_of_nodes*number_of_colours; j++){
		if(j%number_of_nodes*number_of_colours == 0){
			printf("\n"); //new line for each cell
		}
		printf("%d ", cell_list[j]);
	}
	printf("\n");
}

void print_cell_sizes(int *cell_sizes, int length){

	int i=0;
	for(i=0; i<length; i++){
		printf("%d ",cell_sizes[i]);
	}
	printf("\n");
}
