#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include "queue.h"
#include "hash.h"
#include "magic_constants.h"
#include "verbose.h"

#include "assert_helper.h"

static void enqueue_fifo(wQueue * const q, char * const term, const size_t len, const char digest[HASH_LEN]);
static void enqueue_lifo(wQueue * const q, char * const term, const size_t len, const char digest[HASH_LEN]);

/* TODO: disregard the below, we always to BFS right now. (pk, 12.01.2018)
 * initialises a new queue; 
 * if strategy is 0, it is a queue;
 * if strategy is 100, it is a stack;
 * any value in between results in a mixed data structure */
wQueue *init_queue(const int strategy) {
    wQueue *q = (wQueue*) malloc(sizeof(wQueue));
    q->first = NULL;
    q->last = NULL;
    q->c = 0;
    q->strategy = strategy;

    return q;
}

/* enqueue a term with length len and hash code digest into the given queue;
 * function takes ownership of the term */
void enqueue(wQueue *q, char * const term, const size_t len, const char digest[HASH_LEN]) {
      enqueue_fifo(q, term, len, digest);
 //   int r = rand() % 100;
 //   if (r < q->strategy) {
 //       enqueue_lifo(q, term, len, digest);
 //   } else {
 //       enqueue_fifo(q, term, len, digest);
 //   }
}

static void enqueue_fifo(wQueue * const q, char * const term, const size_t len, const char digest[HASH_LEN]) {
    tCell *new = (tCell*) malloc(sizeof(tCell));

    q->c++;

    new->term = term;
    new->term_length = len;
    memcpy(new->digest, digest, HASH_LEN);
    new->next = NULL;

        
    if (q->first == NULL) {
        q->first = new; 
        q->last = new;
        q->first->next = NULL;
    } else { 
        tCell *temp = q->last; // for multithreading
        q->last = new;
        temp->next = new;
    }
}

static void enqueue_lifo(wQueue * const q, char * const term, const size_t len, const char digest[HASH_LEN]) {
    q->c++;

    tCell *new = (tCell*) malloc(sizeof(tCell));
    new->term = term;
    new->term_length = len;
    memcpy(new->digest, digest, HASH_LEN);
    new->next = q->first;

    q->first = new;
    if (q->last == NULL) {
        q->last = new;
    }
}

/* dequeues a cell from given queue;
 * returns NULL, if queue was empty.
 * note that the next pointer will be NULL. */
tCell *dequeue(wQueue * const q) {
    if (q->c == 0) {
        return NULL;
    }
    q->c--;

    tCell *res = q->first;
    assert_es(res->term, "The head of the queue had a NULL term. Aborting.");
    
    q->first = q->first->next;
    
    if (q->first == NULL) {
        q->last = NULL;
    }
    
    res->next = NULL;
    return res;
}

/* returns 1 if the queue is empty,
 * 0 otherwise */
int is_empty(wQueue const * const q) {
    return (q->first == NULL);
}

/* returns the current queue size */
size_t q_size(wQueue const * const q) {
    return q->c;
}

/* tears down a queue and all its queued items */
void teardown_queue(wQueue **q) {
    while((*q)->c > 0) {
        tCell *cell = dequeue(*q);
        free(cell->term);
        free(cell);
    }

    free(*q);
    *q = NULL;
}

/* tears down a cell and its term */
void teardown_cell(tCell **c) {
    assert(c);
    assert(*c);
    assert((*c)->term);
    free((*c)->term);
    free(*c);
    *c = NULL;
}
