#ifndef PID_H_INCLUDED
#define PID_H_INCLUDED

#include <zmq.h>
#include <czmq.h>

struct pid_response {
    unsigned int id;
    zchunk_t *model;
    char *digest_block;
    size_t blocksize;
    char *subscribe_hash_recv_to; // may be NULL, then use master IP
};

/* sets up an IPC socket to serve id requests from workers; 
 * its endpoint will be ipc:///tmp/zmq-proxy-proxynumber-id;
 * returns said socket */
zsock_t *proxy_setup_id_rep_zocket(const int proxynumber);

/* tears down the entire id module */
void proxy_teardown_id(void);

/* receives exactly one id request and replies with an id and a model */
void proxy_h_id(zsock_t * const rep, unsigned int proxy_id, zchunk_t *model);

struct proxy_h_id_args {
    unsigned int proxy_id;
    zchunk_t *model;
};

/* reader function for proxy_h_id */
int proxy_id_reader(zloop_t *reactor, zsock_t *reader, void *arg);

/* requests an id from tcp://master_ip:port,
 * returns the received id and the model */
struct pid_response proxy_request_id(char const * const master_ip,
                                     const int port_req,
                                     const int port_pub,
                                     char const * const own_ip,
                                     const int proxynumber);

zsock_t *proxy_setup_pub_global(const int port_number);

#endif
