#include <assert.h>
#include <string.h>

#include <zmq.h>
#include <czmq.h>

#include <sicstus/sicstus.h>

#include "../common/hash.h"
#include "../common/queue.h"
#include "../common/msg_headers.h"
#include "../common/hashtrie.h"
#include "../common/assert_helper.h"
#include "../common/verbose.h"

#include "wprocessor.h"

zmsg_t *prepare_msg(tCell const * const cell) {
    zmsg_t *msg = zmsg_new();

    char readable[READABLE_HASH_LEN];
    // TODO: eventually get rid of this cast
    print_readable_digest((unsigned char*) cell->digest, readable);

    char processed[HASH_INFO];
    memcpy(processed, cell->digest, HASH_LEN);
    // TODO: for now: every bit is 1 => state is checked
    memset(processed + HASH_LEN, 0xff, INFO_LEN);
    
    zmsg_addstr(msg, readable);
    zmsg_addmem(msg, processed, HASH_INFO);

    return msg;
}

void process_state(tCell const * const cell, const SP_pred_ref process, int fd_logfile) {
    zmsg_t *msg = prepare_msg(cell);

    SP_term_ref term_addr = SP_new_term_ref();
    SP_term_ref msg_ref = SP_new_term_ref();
    SP_term_ref fd_logfile_ref = SP_new_term_ref();

    SP_put_address(term_addr, cell->term);
    SP_put_address(msg_ref, msg);
    SP_put_integer(fd_logfile_ref, fd_logfile);


    int status = SP_query_cut_fail(process, term_addr, msg_ref, fd_logfile_ref);

    // based on in which line the assertion failed, we know the return value
    // yes, this is dirty, but we don't have control about our streams...
    // TODO: signal error to the proxy
    switch(status) {
        case SP_FAILURE:
            assert_es(0, "Prolog answered no.");
            break;
        case SP_ERROR:
            assert_es(0, "Prolog ran into an error.");
            break;
        case SP_SUCCESS:
            break;
    }

}

/* send a small statistic package via zocket.
 * The package consists of:
 *   a header
 *   the worker ID 
 *   the queuesize.
 */
void send_small_stats(zsock_t * const zocket, const int id, const size_t q_size) {
    const int header = IS_SMALL_STATS;
    int rc = zsock_send(zocket, "iib", header, 
                                       id,
                                       &q_size, sizeof(size_t));
    assert(rc == 0);
}


void processor(wQueue * const q,
               zsock_t * const push,
               const int worker_id,
               const SP_pred_ref process_pred,
               const hashtrie trie,
               int fd_logfile) {
    if (is_empty(q)) {
        return;
    }

    tCell *cell = dequeue(q);
    
    if (trie_contains_processed(trie, cell->digest)) {
        if (VERBOSE_WORKER_STATES) {
            char buffi[READABLE_HASH_LEN];
            print_readable_digest((unsigned char*) cell->digest, buffi);
            dprintf(fd_logfile, "{:type :discarding-state-from-queue, :hash :%s}\n", buffi);
        }
        send_small_stats(push, worker_id, q_size(q));
    } else {
        if (VERBOSE_WORKER_STATES) {
            char buffi[READABLE_HASH_LEN];
            print_readable_digest((unsigned char*) cell->digest, buffi);
            dprintf(fd_logfile, "{:type :dequeued, :hash :%s}\n", buffi);
        }
        process_state(cell, process_pred, fd_logfile);
    }
    teardown_cell(&cell);
}

int processor_timer_fn(zloop_t *loop, int timer_id, void *args) {
    struct wprocessor_args *arg = (struct wprocessor_args*) args;
    processor(arg->q, arg->push, arg->worker_id, arg->process_pred, arg->trie, arg->fd_logfile);
    return 0;
}
