% (c) 2009-2019 Lehrstuhl fuer Softwaretechnik und Programmiersprachen,
% Heinrich Heine Universitaet Duesseldorf
% This software is licenced under EPL 1.0 (http://www.eclipse.org/org/documents/epl-v10.html)

:- module(hashing,[my_term_hash/2, efficient_index/1]).

:- use_module(library(terms),[term_hash/3]).


:- use_module(module_information,[module_info/2]).

:- module_info(group,animator).
:- module_info(description,'This module computes hash values (mainly for states) for efficient indexing.').

% --------------------------------------------

:- use_module(self_check).

:- assert_must_succeed(( hashing:my_term_hash([1,2,3],X), hashing:efficient_index(X) )).
:- assert_must_succeed(( hashing:my_term_hash([1,2,3],X), hashing:my_term_hash([1,2,4],Y), X\=Y )).

:- if(tools:platform_is_64_bit).
:- format("64-bit platform~n",[]).
my_term_hash(T,Res) :- super_hash(T,Res).
:- else.
:- format("32-bit platform~n",[]).
my_term_hash(T,Res) :- term_hash(T,
  [range(smallint),  /* will use full 32 bit on 64 bit systems, 28 bit on 32 bit system */
   algorithm(sdbm), % other options jenkins, hsieh, default
   depth(infinite),if_var(ignore)],Res).
% collisions seem with more recent versions of SICStus to no longer be a problem !?
% sdbm seems to generate less collisions
:- endif.

% check if a hash value is an efficient for first argument indexing
efficient_index(X) :- number(X),
  prolog_flag(max_tagged_integer,Max), X=<Max,
  prolog_flag(min_tagged_integer,Min), X>=Min.

% On 64-bit machines we can use the following for efficient indexing:
:- public super_hash/2.
super_hash(Term,Hash) :-
  term_hash(Term,[range(smallint),algorithm(sdbm),depth(infinite),if_var(ignore)],H1),
  term_hash(Term,[range(smallint32),algorithm(jenkins),depth(infinite),if_var(ignore)],H2),
  Hash is H1 + (H2 << 32). % should be in range 0.. 2**60
  % we could still try and use negative numbers

