% (c) 2019-2019 Lehrstuhl fuer Softwaretechnik und Programmiersprachen,
% Heinrich Heine Universitaet Duesseldorf
% This software is licenced under EPL 1.0 (http://www.eclipse.org/org/documents/epl-v10.html)

:- module(kernel_lists,[ not_element_of_list_wf/3, element_of_list_wf/4]).

:- use_module(module_information,[module_info/2]).
:- module_info(group,kernel).
:- module_info(description,'This module provides operations for lists of B objects.').
% with possible repetitions, as, e.g., for set_extensions

:- use_module(kernel_objects,[not_equal_object_wf/3]).
:- use_module(error_manager).

not_element_of_list_wf(Element,List,WF) :- not_el_wf(List,Element,WF).

:- block not_el_wf(-,?,?).
not_el_wf([],_,_WF) :- !.
not_el_wf([H|T],X,WF) :- !,
   not_equal_object_wf(H,X,WF),
   not_el_wf(T,X,WF).
not_el_wf(Other,X,WF) :-
   add_internal_error('Not a list: ',not_el_wf(Other,X,WF)),fail.

% Questions: does it make sense to call clpfd_interface:clpfd_not_inlist ?



% test code below with: i1=1 & j1:0..6 & mid1:0..7 &  mid2 : {mid1/2, mid1}

:- use_module(kernel_mappings,[kernel_call/4,kernel_call_predicate_check_element_of_wf/4]).
:- use_module(clpfd_lists,[try_in_fd_value_list_check/4]).
:- use_module(preferences,[get_preference/2]).

element_of_list_wf(ElementVal,TypeOfElement,List,WF) :-
      % temporary solution: use code from interpreter to evaluate set_extension:
      kernel_call(b_interpreter:convert_list_of_expressions_into_set_wf(List,ValueSet,set(TypeOfElement),WF),
                  List,WF, set_extension(List)), % do we need real AST Expression ? List contains values
      % temporary solution: use check_element_of_wf; to do: use custom version just for lists
      kernel_call_predicate_check_element_of_wf(ElementVal,ValueSet,WF,unknown), %trace,
      %print_term_summary(prop(ExValue,ElementVal,WF)),nl,
      % TO DO: avoid calling check_element_of version when CLPFD complete check possible
      (get_preference(use_clpfd_solver,false) -> true
        ;  get_fd_type_from_b_type(TypeOfElement,FDType),
           try_in_fd_value_list_check(List,ElementVal,FDType,WF)
           % TO DO: provide cleaner API for this
      ),
       tools_printing:print_term_summary(finished_element_of_list_wf(ElementVal,List)),nl.

get_fd_type_from_b_type(integer,T) :- !, T=integer.
get_fd_type_from_b_type(global(G),T) :- !, T=global(G).
get_fd_type_from_b_type(_,_). % otherwise type could be couple_left,...

