% (c) 2019-2019  Lehrstuhl fuer Softwaretechnik und Programmiersprachen,
% Heinrich Heine Universitaet Duesseldorf
% This software is licenced under EPL 1.0 (http://www.eclipse.org/org/documents/epl-v10.html)
:- module(avlp, [
    avl_to_list/2,          % AVL -> List
    is_avl/1,           % AVL ->
    avl_change/5,           % Key -> (AVL x Val <-> AVL x Val)
    avl_domain/2,           % AVL -> OrdSet
    avl_fetch/2,            % Key x AVL ->
    avl_fetch/3,            % Key x AVL -> Val
    avl_height/2,           % AVL -> Height
    avl_incr/4,         % Key x AVL x Inc -> AVL
    avl_max/2,          % AVL -> Key
    avl_max/3,          % AVL -> Key x Val
    avl_member/2,           % Key x AVL
    avl_member/3,           % Key x AVL x Val
    avl_min/2,          % AVL -> Key
    avl_min/3,          % AVL -> Key x Val
    avl_next/3,         % Key x AVL -> Key
    avl_next/4,         % Key x AVL -> Key x Val
    avl_prev/3,         % Key x AVL -> Key
    avl_prev/4,         % Key x AVL -> Key x Val
    avl_range/2,            % AVL -> OrdSet
    avl_size/2,         % AVL -> Size
    avl_store/4,            % Key x AVL x Val -> Val
    avl_delete/4,           % Key x AVL x Val -> AVL
    avl_del_max/4,          % AVL -> Key x Val x AVL
    avl_del_min/4,          % AVL -> Key x Val x AVL
    avl_map/2,          % Goal x AVL ->
    avl_map/3,          % Goal x AVL -> AVL
    empty_avl/1,            % -> AVL
    list_to_avl/2,          % List -> AVL
    ord_list_to_avl/2,      % List -> AVL
    portray_avl/1           % AVL ->
   ]).

:- meta_predicate avl_map(1, +), avl_map(2, +, -).


:- use_module(probsrc(module_information)).
:- module_info(group,tools).
:- module_info(description,'Patched avl library with runtime checks.').


/* plspec setup */
:- use_module(extension('plspec/plspec/plspec_core')).

 :- enable_all_spec_checks.
%:- plspec:enable_spec_check([avl_domain/2, avl_fetch/2]).

:- use_module(library(~~(avl)),
 [is_avl/1, empty_avl/1, portray_avl/1]). % the only predicate we copy as is

% a polymorphic version of the avl_tree type:
:- defspec(avl_tree(X), one_of([compound(node(avl_key,any,integer,avl_tree(X), avl_tree(X))),
                                atom(empty)])).
% just do a top-level check:
%:- defspec(avl_tree, one_of([compound(node(avl_key,any,integer,nonvar, nonvar)), atom(empty)])).
% use built-in from library(avl):
:- defspec_pred(avl_tree, is_avl).

:- defspec_pred(avl_key, ground).
:- use_module(library(~~(ordsets)),[is_ordset/1]).
:- defspec_pred(ordset, is_ordset).

:- spec_pre(avl_to_list/2,  [avl_tree, any]).
:- spec_pre(avl_domain/2,  [avl_tree, any]).
:- spec_pre(avl_fetch/2,  [avl_key, avl_tree]).
:- spec_pre(avl_fetch/3,  [avl_key, avl_tree, any]).
:- spec_pre(avl_height/2,  [avl_tree, one_of([var, integer])]).
:- spec_pre(avl_max/2,  [avl_tree, one_of([var, any])]).
:- spec_pre(avl_max/3,  [avl_tree, one_of([var, avl_key]), any]).
:- spec_pre(avl_min/2,  [avl_tree, one_of([var, any])]).
:- spec_pre(avl_min/2,  [avl_tree, one_of([var, avl_key, any])]).
:- spec_pre(avl_member/2,  [any ,avl_tree]).
:- spec_pre(avl_member/3,  [any ,avl_tree, any]).

:- spec_pre(avl_next/3,  [avl_key, avl_tree, any]).
:- spec_pre(avl_next/4,  [avl_key, avl_tree, any, any]).
:- spec_pre(avl_prev/3,  [avl_key, avl_tree, any]).
:- spec_pre(avl_prev/4,  [avl_key, avl_tree, any, any]).

:- spec_pre(avl_range/2,  [avl_tree, any]).
:- spec_pre(avl_size/2,  [avl_tree, one_of([var, integer])]).


:- spec_pre(avl_store/4,  [avl_key, avl_tree, any, any]).
:- spec_pre(avl_delete/4,  [avl_key, avl_tree, any, any]).
:- spec_pre(avl_del_max/4,  [avl_tree, any, any, any]).
:- spec_pre(avl_del_min/4,  [avl_tree, any, any, any]).

:- spec_pre(ord_list_to_avl/2,  [ordset, any]).

:- spec_pre(avl_change/5,  [avl_key,avl_tree, any, any, any]).

avl_to_list(A,B) :- avl:avl_to_list(A,B).
avl_domain(A,B) :- avl:avl_domain(A,B).
avl_fetch(A,B) :- avl:avl_fetch(A,B).
avl_fetch(A,B,C) :- avl:avl_fetch(A,B,C).
avl_height(A,B) :- avl:avl_height(A,B).
avl_incr(A,B,C,D) :- avl:avl_incr(A,B,C,D).
avl_max(A,B) :- avl:avl_max(A,B).
avl_max(A,B,C) :- avl:avl_max(A,B,C).
avl_member(A,B) :- avl:avl_member(A,B).
avl_member(A,B,C) :- avl:avl_member(A,B,C).
avl_min(A,B) :- avl:avl_min(A,B).
avl_min(A,B,C) :- avl:avl_min(A,B,C).

avl_next(A,B,C) :- avl:avl_next(A,B,C).
avl_next(A,B,C,D) :- avl:avl_next(A,B,C,D).
avl_prev(A,B,C) :- avl:avl_prev(A,B,C).
avl_prev(A,B,C,D) :- avl:avl_prev(A,B,C,D).
avl_range(A,B) :- avl:avl_range(A,B).
avl_size(A,B) :- avl:avl_size(A,B).

avl_store(A,B,C,D) :- avl:avl_store(A,B,C,D).
avl_delete(A,B,C,D) :- avl:avl_delete(A,B,C,D).
avl_del_max(A,B,C,D) :- avl:avl_del_max(A,B,C,D).
avl_del_min(A,B,C,D) :- avl:avl_del_min(A,B,C,D).

avl_map(A,B) :- avl:avl_map(A,B).
avl_map(A,B,C) :- avl:avl_map(A,B,C).
list_to_avl(A,B) :- avl:list_to_avl(A,B).
ord_list_to_avl(A,B) :- avl:ord_list_to_avl(A,B).

avl_change(Node,Vin,OldValue,Vout,NewValue) :- avl:avl_change(Node,Vin,OldValue,Vout,NewValue).
