% (c) 2014-2019 Lehrstuhl fuer Softwaretechnik und Programmiersprachen,
% Heinrich Heine Universitaet Duesseldorf
% This software is licenced under EPL 1.0 (http://www.eclipse.org/org/documents/epl-v10.html)

:- module(dot_graphs_static_analysis, [tcltk_create_dependence_graph/1, tcltk_create_enable_graph/1,tcltk_create_enable_graph_por/3,tcltk_create_disable_graph_por/2]).

/* Modules and Infos for the code coverage analysis */
:- use_module(probsrc(module_information)).
:- module_info(group,model_checker).
:- module_info(description,'This module provides predicates for printing out graphs in dot-format calculated by computing some of the static analyses used for partial order reduction').

% Standard SICSTUS prolog libraries
:- use_module(library(lists)).

% Imports from the POR modules
:- use_module(probporsrc(static_analysis),[dependent_actions/5,enable_analysis/6,disable_analysis/5]).

% The dot-graph generator
:- use_module(probsrc(dot_graph_generator),[gen_dot_graph/6]).

% Preferences
:- use_module(probsrc(preferences), [get_preference/2]).

% TODO: move all enabling graph predicates in enabling_analysis.pl used for generating dot-graphs to a new module
:- use_module(probsrc(enabling_analysis), [translate_enable_res/6, eop_node_predicate/6, cbc_enable_analysis/4]).

% Predicates for creating an enable graphs
tcltk_create_enable_graph(File) :-
  gen_dot_graph(File,dot_graphs_static_analysis,eop_node_predicate,cbc_enable_trans_predicate,none,none).

tcltk_create_enable_graph_por(File,Inv,UseEnableGraph) :-
  EnablePred = cbc_enable_trans_predicate_por(Inv,UseEnableGraph),
  gen_dot_graph(File,dot_graphs_static_analysis,enable_op_node_predicate,EnablePred,none,none).

tcltk_create_disable_graph_por(File,Inv) :-
  DisablePred = cbc_disable_trans_por(Inv),
  gen_dot_graph(File,dot_graphs_static_analysis,enable_op_node_predicate,DisablePred,none,none).

:- public enable_op_node_predicate/6.
enable_op_node_predicate(NodeID,SubGraph,NodeDesc,Shape,Style,Color) :-
  eop_node_predicate(NodeID,SubGraph,NodeDesc,Shape,Style,Color),
  NodeID \='INITIALISATION'.

:- public cbc_disable_trans_por/6.
cbc_disable_trans_por(Inv,NodeID,Label,SuccID,Color,Style) :-
  get_preference(timeout_cbc_analysis,Timeout),
  disable_analysis(NodeID,SuccID,Inv,Timeout,Res),
  (Res=possible_disable; Res=timeout),
  translate_enable_res(Res,NodeID,SuccID,Label,Color,Style).

:- public cbc_enable_trans_predicate_por/7.
cbc_enable_trans_predicate_por(Inv,UseEnableGraph,NodeID,Label,SuccID,Color,Style) :-
  get_preference(timeout_cbc_analysis,Timeout),
  enable_analysis(NodeID,SuccID,Inv,UseEnableGraph,Timeout,Res),
  (Res=possible_enable; Res=possible; Res=guaranteed; Res=timeout; Res=predicate(_Expr)),
  translate_enable_res(Res,NodeID,SuccID,Label,Color,Style).

:- public cbc_enable_trans_predicate/5.
cbc_enable_trans_predicate(NodeID,Label,SuccID,Color,Style) :-
  cbc_enable_analysis(NodeID,SuccID,Res,500),
  translate_enable_res(Res,NodeID,SuccID,Label,Color,Style).

% Predicates for creating a dependency graph
tcltk_create_dependence_graph(File) :-
  gen_dot_graph(File,dot_graphs_static_analysis,enable_op_node_predicate,cbc_dependence_trans_predicate,none,none).

:- public cbc_dependence_trans_predicate/5.
cbc_dependence_trans_predicate(NodeID,Label,SuccID,Color,Style) :-
  Inv=0, % TODO: remove this later
  get_preference(timeout_cbc_analysis,Timeout),
  dependent_actions(NodeID,SuccID,Inv,Timeout,Res),
  (Res=dependent; Res=race_dependent),
  translate_enable_res(Res,NodeID,SuccID,Label,Color,Style).
