:- module(solver_dispatcher, [init_interface/1,reset_interface/1,smt_solver_interface_call/2,
                              pretty_print_smt/1, pretty_print_smt/2]).

:- use_module(probsrc(module_information),[module_info/2]).
:- module_info(group,smt_solvers).
:- module_info(description,'This module provides an interface to SMT-solvers like CVC4 or Z3.').

:- use_module(extension('cvc4interface/cvc4interface')).
:- use_module(extension('z3interface/z3interface')).

:- use_module(probsrc(debug), [debug_format_flush/3]).

init_interface(cvc4) :-
    init_cvc4interface.
init_interface(z3) :-
    init_z3interface.
reset_interface(cvc4) :-
    reset_cvc4interface.
reset_interface(z3) :-
    reset_z3interface.

:- meta_predicate smt_solver_interface_call(+,+).
smt_solver_interface_call(z3,Call) :-
    %debug_format(9,'z3: calling ~w~n',[Call]),
    z3_interface_call(Call),
    debug_format_flush(9,'z3: called ~w~n',[Call]).
smt_solver_interface_call(cvc4,Call) :-
    debug_format_flush(9,'cvc4: calling ~w~n',[Call]),
    cvc4_interface_call(Call).

pretty_print_smt(z3) :- !,pretty_print_smt2.
pretty_print_smt(Solver) :- format('Pretty print not available for ~w~n',[Solver]).

pretty_print_smt(z3,ExprID) :- !,pretty_print_smt2_for_id(ExprID).
pretty_print_smt(Solver,_) :- format('Pretty print not available for ~w~n',[Solver]).
