% (c) 2018-2019 Lehrstuhl fuer Softwaretechnik und Programmiersprachen,
% Heinrich Heine Universitaet Duesseldorf
% This software is licenced under EPL 1.0 (http://www.eclipse.org/org/documents/epl-v10.html)

:- module(source_profiler,[reset_source_profiler/0, source_profiler_enabled/0,
                            add_source_location_hits/2,
                            %tcltk_get_source_profile_info/1,
                            show_source_profile_in_bbresults/0,
                            print_source_profile/0,
                            tcltk_get_source_hit_location/5]).

% TO DO: optionally load more precise microsecond timer

:- load_files(library(system), [when(compile_time), imports([environ/2])]).

:- if(\+ environ(prob_src_profile, true)).

source_profiler_enabled :- fail.
reset_source_profiler.
add_source_location_hits(_,_).
print_source_profile :-
    print('No source profiling information available'),nl,
    print('Recompile ProB with -Dprob_src_profile=true'),nl.
show_source_profile_in_bbresults.
tcltk_get_source_hit_location(_,_,_,_,_) :- fail.
:- else.

:- volatile source_hits/3.
:- dynamic source_hits/3.

source_profiler_enabled.
reset_source_profiler :-
  retractall(source_hits(_,_,_)).

:- use_module(eventhandling,[register_event_listener/3]).
:- register_event_listener(clear_specification,reset_source_profiler,
                           'Reset source location profiler.').

:- use_module(library(terms),[term_hash/2]).
add_source_location_hits(SourceSpan,Nr) :-
  %extract_span_description(SourceSpan,M), print(M),nl,
  term_hash(SourceSpan,Hash),
  (retract(source_hits(Hash,SourceSpan,Old)) -> New is Old+Nr ; New is Nr),
  assert(source_hits(Hash,SourceSpan,New)).

:- use_module(library(lists)).
:- use_module(error_manager,[extract_span_description/2, extract_file_line_col/6, extract_line_col_for_main_file/5]).

print_source_profile :- format('----Source Location Profiler Info----~n',[]),
   findall(src_loc_msg(Nr,FullFilename,Line,Col,EndLine,EndCol),
           source_hit(FullFilename,Line,Col,EndLine,EndCol,Nr),
           Ls),
    sort(Ls,Sorted),
    maplist(print_src,Sorted),
    format('----~n',[]).

:- use_module(tools_commands,[show_source_locations_with_bb_results/1]).
show_source_profile_in_bbresults :-
   findall(src_loc_msg(Nr,FullFilename,Line,Col,EndLine,EndCol),
           source_hit(FullFilename,Line,Col,EndLine,EndCol,Nr),
           Ls),
    sort(Ls,Sorted), reverse(Sorted,RS),
    show_source_locations_with_bb_results(RS).

source_hit(FullFilename,Line,Col,EndLine,EndCol,Nr) :-
   source_hits(_,SourceSpan,Nr),
   extract_file_line_col(SourceSpan,FullFilename,Line,Col,EndLine,EndCol).

print_src(src_loc_msg(Nr,FullFilename,Line,Col,EndLine,EndCol)) :-
  format(' ~w hits at ~w:~w -- ~w:~w in ~w~n',[Nr,Line,Col,EndLine,EndCol,FullFilename]).

% can be used in a while loop:
tcltk_get_source_hit_location(Nr,Srow,Scol,Erow,Ecol) :-
  retract(source_hits(_,Span,Nr)),
  extract_line_col_for_main_file(Span,Srow,Scol,Erow,Ecol).

:- endif.
