% (c) 2009-2019 Lehrstuhl fuer Softwaretechnik und Programmiersprachen,
% Heinrich Heine Universitaet Duesseldorf
% This software is licenced under EPL 1.0 (http://www.eclipse.org/org/documents/epl-v10.html)


:- module(tools_timeout, [
                  time_out_call/2, time_out_call/1,
                  time_out_with_factor_call/3, time_out_with_factor_call/4,
                  get_time_out_with_factor/2
                  ]).

:- use_module(module_information).

:- module_info(group,infrastructure).
:- module_info(description,'This module contains higher-level timeout helper predicates.').


:- meta_predicate time_out_with_factor_call(0,*,0).
:- meta_predicate time_out_call(0,0).
:- meta_predicate time_out_call(0).

:- use_module(tools_meta,[safe_time_out/3]).
:- use_module(tools_printing,[print_term_summary/1]).
:- use_module(error_manager,[add_message/3, add_error_and_fail/3]).
:- use_module(tools_strings,[predicate_functor/3]).
:- use_module(preferences,[get_preference/2]).

time_out_call(Call,Res) :-
  get_preference(time_out,CurTO),
  safe_time_out(Call,CurTO,TimeOutRes),
  (TimeOutRes=time_out
    -> predicate_functor(Call,F,N),
       add_message(self_check,'Time out occurred: ',F/N),
       print_term_summary(Call),
       call(Res)
    ;  true).

time_out_call(Call) :-
   time_out_call(Call,add_error_and_fail(time_out_call,'*** TIMEOUT occurred: ',Call)).

:- use_module(preferences,[get_time_out_preference_with_factor/2]).
get_time_out_with_factor(fixed_time_out(Fixedms),TORes) :- % also allow to specify a hard time out value
  !, TORes = Fixedms.
get_time_out_with_factor(Factor,Res) :-
  get_time_out_preference_with_factor(Factor,Res).

time_out_with_factor_call(Call,Factor,Res) :-
  time_out_with_factor_call(Call,Factor,[],Res).
time_out_with_factor_call(Call,Factor,Options,Res) :-
  get_time_out_with_factor(Factor,TO),
  %%print(time_out(Call,TO)),nl,
  statistics(runtime,[Start,_]),
  on_exception(enumeration_warning(_,_,_,_,_),
               safe_time_out(Call,TO,TimeOutRes),
               (TimeOutRes = time_out,EnumWarning=true)),
  (TimeOutRes=time_out
    -> (member(silent,Options) -> true
        ;  statistics(runtime,[Stop,_]),
           predicate_functor(Call,F,N),
           (EnumWarning==true
             -> add_message(self_check,'Time-out forced by enumeration warning: ',F/N)
              ; add_message(self_check,'Time-out occurred: ',F/N)
            ),
           print_term_summary(Call),
           Diff is Stop-Start,
           print(Diff), print(' ms runtime (time_out = '), print(TO), print(' ms)'),nl
        ),
       call(Res)
    ;  true).
